package com.as.lyons.res.svc.aoa.client;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.web.client.RestTemplate;

import com.as.lyons.rest.svc.aoa.AccountOwnerDetailsRequest;


/**
 * Sample AOA REST Service client
 *
 */
@SpringBootApplication
public class App implements LogonProperties {
	private static final Logger log = LoggerFactory.getLogger(App.class);

	private static final Map<String, AoaServiceCall> svcMethods = new HashMap<>();

	private static String appStartCommand = "java -cp <...> <your class> ";

	public static void main(String[] args) {
		setAppStartCommand();
		svcMethods.put("help", new HelpAoaServiceCall());
		svcMethods.put("CheckAccountOwnershipAndStatus", new CheckAccountOwnershipAndStatusCall());
		svcMethods.put("CheckAccountStatus", new CheckAccountStatusServiceCall());
		svcMethods.put("CheckAccountOwnership", new CheckAccountOwnershipCall());
		svcMethods.put("CheckAccountOwnershipAndStatusWithDates", new CheckAccountOwnershipAndStatusWithDatesCall());
		svcMethods.put("CheckAccountStatusWithDates", new CheckAccountStatusServiceWithDatesCall());
		svcMethods.put("CheckAccountOwnershipWithDates", new CheckAccountOwnershipWithDatesCall());
		svcMethods.put("DescribeStatusCode", new DescribeStatusCodeServiceCall());
		SpringApplication.run(App.class, args);
	}

	@Value("${base.service.url}")
	private String baseUrl;
	@Value("${company.id}")
	private int companyId;
	@Value("${company.user.name}")
	private String userName;
	@Value("${company.user.password}")
	private String password;
	@Value("${owner.firstName}")
	private String firstName;
	@Value("${owner.lastName}")
	private String lastName;
	@Value("${owner.middleName}")
	private String middleName;
	@Value("${owner.namePfx}")
	private String namePfx;
	@Value("${owner.nameSfx}")
	private String nameSfx;
	@Value("${owner.businessName}")
	private String businessName;
	@Value("${owner.address1}")
	private String address1;
	@Value("${owner.address2}")
	private String address2;
	@Value("${owner.city}")
	private String city;
	@Value("${owner.state}")
	private String state;
	@Value("${owner.zip}")
	private String zip;
	@Value("${owner.ssn}")
	private String ssn;
	@Value("${owner.dob}")
	private String dob;
	@Value("${owner.idType}")
	private String idType;
	@Value("${owner.idNo}")
	private String idNo;
	@Value("${owner.idState}")
	private String idState;
	private static final AccountOwnerDetailsRequest owner = new AccountOwnerDetailsRequest();

	@Bean
	public RestTemplate restTemplate(RestTemplateBuilder builder, @Value("${base.service.url}") String localBaseUrl) {
		return builder.rootUri(localBaseUrl).messageConverters(new MappingJackson2HttpMessageConverter()).build();
	}

	@Bean
	public CommandLineRunner run(RestTemplate restTemplate) throws Exception {
		final LogonProperties logonProps = this;
		return args -> {
			log.info("Begin method execution");
			System.out.println("\n\n\n");
			String[] augArgs = new String[0];
			String methodName = null;
			if (args != null) {
				List<String> argLst = new ArrayList<>(args.length);
				for (String s : args) {
					if (s != null && !s.startsWith("--")) {
						if (methodName == null) {
							methodName = s;
						} else {
							argLst.add(s);
						}
					}
				}
				augArgs = argLst.toArray(augArgs);
			}
			if (augArgs.length == 0) {
				printHelp("Invalid number of arguments provided!");
				return;
			}
			AoaServiceCall method = svcMethods.get(methodName);
			if (method == null) {
				printHelp("The provided method is not recognized!");
				return;
			}
			try {
				method.call(restTemplate, logonProps, methodName, augArgs);
			} catch (StopProcesingException e) {
			}
			System.out.println("\n\n\n");
			log.info("End method execution");
		};
	}

	protected void printHelp(String error) {
		if (error != null) {
			System.out.println(error);
		}
		System.out.println(new StringBuilder(500).append("Use:\n\t").append(appStartCommand)
				.append("help <method name>").toString());
		System.out.println(new StringBuilder(500).append("\nor:\n\t").append(appStartCommand)
				.append("<method name> <arguments>").toString());
		System.out.println("\twhere <method name> can be:");
		for (String method : svcMethods.keySet()) {
			if (!"help".equals(method)) {
				System.out.println("\t\t".concat(method));
			}
		}
		System.out.println("\n\n\n");
	}

	@Override
	public int getCompanyId() {
		return companyId;
	}

	@Override
	public String getUserName() {
		return userName;
	}

	@Override
	public String getPassword() {
		return password;
	}

	@Override
	public String getAppStartCommand() {
		return appStartCommand;
	}

	@Override
	public Map<String, AoaServiceCall> getSvcMethods() {
		return Collections.unmodifiableMap(svcMethods);
	}

	private static void setAppStartCommand() {
		ClassLoader loader = App.class.getClassLoader();
		if (loader == null) {
			// Try the bootstrap classloader - obtained from the ultimate parent
			// of the System Class Loader.
			loader = ClassLoader.getSystemClassLoader();
			while (loader != null && loader.getParent() != null) {
				loader = loader.getParent();
			}
		}
		if (loader == null) {
			return;
		}
		String name = App.class.getCanonicalName();
		URL resource = loader.getResource(name.replace(".", "/") + ".class");
		if (resource == null) {
			return;
		}

		String resStr = resource.toString();
		String jar = null;
		if (resStr.startsWith("jar:")) {
			int jarIdx = resStr.indexOf(".jar!");
			if (jarIdx > 0) {
				resStr = resStr.substring(0, jarIdx + 4);
				jarIdx = resStr.lastIndexOf('/');
				if (jarIdx >= 0) {
					jar = resStr.substring(jarIdx + 1);
				} else {
					jar = resStr;
				}
			}
		}
		StringBuilder sb = new StringBuilder(500).append("java ");
		if (jar == null) {
			sb.append("-cp <...> ").append(name);
		} else {
			sb.append("-jar ").append(jar);
		}
		appStartCommand = sb.append(" ").toString();
	}
	
	public void setOwner() {
		  owner.setFirstName(firstName);
		  owner.setLastName(lastName);
		  owner.setMiddleName(middleName);
		  owner.setNamePfx(namePfx);
		  owner.setNameSfx(nameSfx);
		  owner.setBusinessName(businessName);
		  owner.setAddress1(address1);
		  owner.setAddress2(address2);
		  owner.setCity(city);
		  owner.setState(state);
		  owner.setZip(zip);
		  owner.setSsn(ssn);
		  owner.setDob(dob);
		  owner.setIdType(idType);
		  owner.setIdNo(idNo);
		  owner.setIdState(idState);
		  
		}
		@Override
		public AccountOwnerDetailsRequest getOwner() {
			setOwner();
			return owner;
		}
}
